@@ -525,12 +525,14 @@ load_dir (GtkFileFolderGnomeVFS *folder_vfs)
 
   g_hash_table_foreach_remove (folder_vfs->children,
 			       remove_all, NULL);
+  gnome_authentication_manager_push_async ();
   gnome_vfs_async_load_directory (&folder_vfs->async_handle,
 				  folder_vfs->uri,
 				  get_options (folder_vfs->types),
 				  ITEMS_PER_NOTIFICATION,
 				  GNOME_VFS_PRIORITY_DEFAULT,
 				  directory_load_callback, folder_vfs);
+  gnome_authentication_manager_pop_async ();
 }
 
 static char *
@@ -585,10 +587,12 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 
   folder_vfs = g_object_new (GTK_TYPE_FILE_FOLDER_GNOME_VFS, NULL);
 
+  gnome_authentication_manager_push_sync ();
   result = gnome_vfs_monitor_add (&monitor,
 				  uri,
 				  GNOME_VFS_MONITOR_DIRECTORY,
 				  monitor_callback, folder_vfs);
+  gnome_authentication_manager_pop_sync ();
   if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_NOT_SUPPORTED)
     {
       g_free (uri);
@@ -627,9 +631,11 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 	  FolderChild *child;
 
 	  vfs_info = gnome_vfs_file_info_new ();
+	  gnome_authentication_manager_push_sync ();
 	  result = gnome_vfs_get_file_info (folder_vfs->uri,
 					    vfs_info,
 					    get_options (parent_folder->types));
+	  gnome_authentication_manager_pop_sync ();
 	  if (result != GNOME_VFS_OK)
 	    {
 	      gnome_vfs_file_info_unref (vfs_info);
@@ -661,10 +667,12 @@ gtk_file_system_gnome_vfs_create_folder (GtkFileSystem     *file_system,
   const gchar *uri = gtk_file_path_get_string (path);
   GnomeVFSResult result;
 
+  gnome_authentication_manager_push_sync ();
   result = gnome_vfs_make_directory (uri,
 				     GNOME_VFS_PERM_USER_ALL |
 				     GNOME_VFS_PERM_GROUP_ALL |
 				     GNOME_VFS_PERM_OTHER_READ);
+  gnome_authentication_manager_pop_sync ();
 
   if (result != GNOME_VFS_OK)
     {
@@ -764,7 +772,9 @@ gtk_file_system_gnome_vfs_volume_mount (GtkFileSystem        *file_system,
 
       closure.system_vfs = system_vfs;
       closure.loop = g_main_loop_new (NULL, FALSE);
+      gnome_authentication_manager_push_sync ();
       gnome_vfs_drive_mount (GNOME_VFS_DRIVE (volume), volume_mount_cb, &closure);
+      gnome_authentication_manager_pop_sync ();
 
       GDK_THREADS_LEAVE ();
       g_main_loop_run (closure.loop);
@@ -1078,11 +1088,9 @@ gtk_file_system_gnome_vfs_parse (GtkFileSystem     *file_system,
   gboolean result = FALSE;
 
   /* Strip off leading whitespaces (since they can't be part of a valid
-   * uri). Strip off trailing whitespaces too because filenames with
-   * trailing whitespace are crack (We want to avoid letting the
-   * user accidentally create such filenames for the save dialog.)
+   * uri).
    */
-  stripped = g_strstrip (g_strdup (str));
+  stripped = g_strchug (g_strdup (str));
 
   last_slash = strrchr (stripped, '/');
   if (!last_slash)
@@ -1271,25 +1279,30 @@ get_vfs_info (GtkFileSystem     *file_system,
     return NULL;
   
   parent_uri = gtk_file_path_get_string (parent_path);
-  canonical_uri = make_uri_canonical (parent_uri);
-  folder_vfs = g_hash_table_lookup (system_vfs->folders, canonical_uri);
-  g_free (canonical_uri);
   info = NULL;
-  if (folder_vfs &&
-      (folder_vfs->types & types) == types)
+  if (parent_uri != NULL)
     {
-      info = lookup_vfs_info_in_folder (GTK_FILE_FOLDER (folder_vfs),
-					path,
-					NULL);
-      if (info)
-	gnome_vfs_file_info_ref (info);
+      canonical_uri = make_uri_canonical (parent_uri);
+      folder_vfs = g_hash_table_lookup (system_vfs->folders, canonical_uri);
+      g_free (canonical_uri);
+      if (folder_vfs &&
+	  (folder_vfs->types & types) == types)
+	{
+	  info = lookup_vfs_info_in_folder (GTK_FILE_FOLDER (folder_vfs),
+					    path,
+					    NULL);
+	  if (info)
+	    gnome_vfs_file_info_ref (info);
+	}
     }
   
   if (info == NULL)
     {
       info = gnome_vfs_file_info_new ();
       uri = gtk_file_path_get_string (path);
+      gnome_authentication_manager_push_sync ();
       gnome_vfs_get_file_info (uri, info, get_options (types));
+      gnome_authentication_manager_pop_sync ();
     }
   
   gtk_file_path_free (parent_path);
@@ -1793,7 +1806,9 @@ lookup_vfs_info_in_folder (GtkFileFolder     *folder,
       GnomeVFSResult result;
       
       vfs_info = gnome_vfs_file_info_new ();
+      gnome_authentication_manager_push_sync ();
       result = gnome_vfs_get_file_info (uri, vfs_info, get_options (folder_vfs->types));
+      gnome_authentication_manager_pop_sync ();
       
       if (result != GNOME_VFS_OK)
 	set_vfs_error (result, uri, error);
@@ -1826,6 +1841,34 @@ gtk_file_folder_gnome_vfs_get_info (GtkFileFolder     *folder,
   GtkFileFolderGnomeVFS *folder_vfs = GTK_FILE_FOLDER_GNOME_VFS (folder);
   const gchar *uri = gtk_file_path_get_string (path);
   GnomeVFSFileInfo *info;
+
+  if (!path)
+    {
+      GnomeVFSURI *vfs_uri;
+      GnomeVFSResult result;
+      GtkFileInfo *file_info;
+
+      vfs_uri = gnome_vfs_uri_new (folder_vfs->uri);
+      g_assert (vfs_uri != NULL);
+
+      g_return_val_if_fail (!gnome_vfs_uri_has_parent (vfs_uri), NULL);
+      gnome_vfs_uri_unref (vfs_uri);
+
+      info = gnome_vfs_file_info_new ();
+      gnome_authentication_manager_push_sync ();
+      result = gnome_vfs_get_file_info (folder_vfs->uri, info, get_options (GTK_FILE_INFO_ALL));
+      gnome_authentication_manager_pop_sync ();
+      if (result != GNOME_VFS_OK)
+	{
+	  file_info = NULL;
+	  set_vfs_error (result, folder_vfs->uri, error);
+	}
+      else
+	file_info = info_from_vfs_info (folder_vfs->uri, info, GTK_FILE_INFO_ALL);
+
+      gnome_vfs_file_info_unref (info);
+      return file_info;
+    }
   
   info = lookup_vfs_info_in_folder (folder, path, error);
 
@@ -1842,7 +1885,7 @@ list_children_foreach (gpointer key,
 {
   FolderChild *child = value;
   GSList **list = user_data;
-
+  
   *list = g_slist_prepend (*list, gtk_file_path_new_dup (child->uri));
 }
 
@@ -2093,7 +2136,9 @@ monitor_callback (GnomeVFSMonitorHandle   *handle,
 	GnomeVFSFileInfo *vfs_info;
 
 	vfs_info = gnome_vfs_file_info_new ();
+	gnome_authentication_manager_push_sync ();
 	result = gnome_vfs_get_file_info (info_uri, vfs_info, get_options (folder_vfs->types));
+	gnome_authentication_manager_pop_sync ();
 
 	if (result == GNOME_VFS_OK)
 	  {
